/* Generic routines for debugging and memory management*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include "stuff.h"
#define MAGIC_NUMBER 0xB0F0CAFE
#define SHRED_VALUE 0x00
#ifdef KEEP_MALLOC_STATS
static size_t alloc_count=0;
static size_t alloc_bytes=0;
static size_t free_count=0;
#endif

void __assert__(int x,char *fn,int line)
{
#ifdef DEBUG
	if (!x)
	{
		printf("Assertion Failed\nFile:%s\nLine:%i\n",fn,line);
		abort(); /* dump core */
	}
#endif
}

int expand_str(char **str)
{
	int len,new_len;
	char *new_str;
	ASSERT(str!=NULL);
	ASSERT((*str)!=NULL);
	len=strlen(*str);
	new_len=2*len;
	new_str=fn_realloc(*str,new_len);
	if (new_str!=NULL)
	{
		*str=new_str;
		return new_len;
	}else
	{
		return -1;
	}		
}
int isalpha(char d)
{
	if (d>='a') if (d<='z') return 1;
    if (d>='A') if (d<='Z') return 1;
    if (d=='_') return 1;
    return 0;
}

int isnum(char d)
{
    return ((d>='0')&&(d<='9'))?1:0;
}

int isalphanum(char d)
{
	return (isalpha(d)||isnum(d));
}


/* same as strdup but uses the malloc wrapper
 * function */
char * duplicate_str(char *string)
{
	char * result;
	size_t len;
	ASSERT (string != NULL);
	if (!string)
		return NULL;
	len = strlen(string);
	if (len == 0)
		return NULL;
	result = (char *)fn_malloc(len+1);
	memcpy(result,string,len);
	result[len]=0;
	return result;
}

/*chars that can't be found in a valid Unix file name*/
#define INVALID_CHAR_COUNT 10 
const char invalid_chars[INVALID_CHAR_COUNT]={'&','/','\\','|',
	'<','>',',','\'','"','?'};

int is_invalid_char(char ch)
{
	int i=INVALID_CHAR_COUNT;
	while (--i>0)
		if (invalid_chars[i]==ch)return 1;
	return -1;
			
}

char *convert_url_to_file(char *url)
{
	size_t file_len;
	int offset,i;
	static char file[1024];
	offset=0;
	if (strncmp(url,"http://",7)==0)
			offset=7;
	file_len=strlen(url)-offset;
	file[file_len]=0;
	for(i=0;i<file_len;++i)
	{
		file[i]=*(offset+ url++);
		if (is_invalid_char(file[i])>0)
			file[i]='.';
	}
	return file;
}

/*stupid hack to get the size of a file
  or verify that it exists*/
int get_file_size(char *fname)
{
	int r;
	FILE *s;
	if (!fname)
		return -1;
	s=fopen(fname,"rb");
	if (!s)return -1;
 	fseek(s,0,SEEK_END);
	r=ftell(s);
 	fclose(s);
 	return r;
}

void save_file(char *buffer,char *filename,size_t size)
{
	FILE *stream=fopen(filename,"a");
	if (!stream)
	{
		fprintf(stderr,"Unable to write file:%s\n",filename);
		return;
	}
	fwrite(buffer,1,size,stream);
	fclose(stream);
}
char *read_file(char *filename)
{
	long i;
	FILE *stream;
	char *buffer;
	i=get_file_size(filename);
// 	printf("file_size:%s:%i\n",filename,i);
	if (i<=0)
		return (char *)NULL;
	stream=fopen(filename,"rb");
	if (!stream)
			return (char *)NULL;
	buffer=(char *)fn_malloc(i+1);
	fread(buffer,1,i,stream);
	fclose(stream);
	buffer[i]=0;
	return buffer;
}

#ifdef DEBUG
void _fn_free(void *ptr,char *file,int line)
{
	if (ptr==NULL)
	{
		fprintf(stderr, "NULL pointer passed to free()\n");
		fprintf(stderr, "File:%s\nLine:%i\n",file,line);
		exit(-1);
	}
#else
void _fn_free(void *ptr)
{
#endif
	if (ptr!=NULL)
	{
		free(ptr);
#ifdef KEEP_MALLOC_STATS
	free_count++;
#endif
	}
}

#ifdef DEBUG
void * _fn_malloc(size_t size,char *file,int line)
#else
void * _fn_malloc(size_t size)
#endif
{
	void *tmp=malloc(size);
	if (!tmp)
	{
		fprintf(stderr, "Unable to allocate enough memory\n");
#ifdef DEBUG
		fprintf(stderr, "Memory Allocation Failed:%s,Line %i\n",file,line);
#endif
		exit(-1);
	}
#ifdef KEEP_MALLOC_STATS
	alloc_count++;
	alloc_bytes+=size;
#endif
	return tmp;
}
#ifdef DEBUG
void *_fn_realloc(void *ptr,size_t size,char *file,int line)
#else
void *_fn_realloc(void *ptr,size_t size)
#endif
{
	void *tmp;
	tmp=realloc(ptr,size);
	if (!tmp)
	{
		fprintf (stderr, "Unable to reallocate memory\n");
#ifdef DEBUG
		fprintf(stderr, "Reallocation failed:%s, Line %i\n",file,line);
#endif
		exit (-1);
	}
	return tmp;
}

void print_stats()
{
#ifdef KEEP_MALLOC_STATS
	printf("Allocated Bytes:%i\nAllocated Chunks:%i\nFreed Chunks:%i\n",
			alloc_bytes,alloc_count,free_count);
#endif
}
